import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { Component } from 'react';
const DefaultLoader = () => null;
const loadingStates = new WeakMap();
const getLoadingState = (loadFn, modify) => {
    if (!loadingStates.has(loadFn)) {
        loadingStates.set(loadFn, {
            error: null,
            loading: true,
        });
    }
    const loadingState = loadingStates.get(loadFn);
    if (modify == null) {
        return loadingState;
    }
    const updatedState = { ...loadingState, ...modify };
    loadingStates.set(loadFn, updatedState);
    return updatedState;
};
const load = (loadFn) => {
    const state = getLoadingState(loadFn);
    if (state.result) {
        return state;
    }
    getLoadingState(loadFn, { loading: true, error: null });
    state.loading = true;
    state.error = null;
    state.promise = loadFn()
        .then((result) => {
        state.loading = false;
        state.result = result;
        getLoadingState(loadFn, state);
        return result;
    })
        .catch((error) => {
        state.loading = false;
        state.error = error;
        getLoadingState(loadFn, state);
        throw error;
    });
    getLoadingState(loadFn, state);
    return state;
};
class LoadableComponentBase extends Component {
    config;
    constructor(config, props) {
        super(props);
        this.config = config;
        this.state = getLoadingState(config.loadFn);
    }
    static loadFn = undefined;
    static preload() {
        load(this.loadFn);
    }
    UNSAFE_componentWillMount() {
        if (!this.state.loading) {
            return;
        }
        const { promise } = load(this.config.loadFn);
        this.setState(getLoadingState(this.config.loadFn));
        promise.finally(() => {
            this.setState(getLoadingState(this.config.loadFn));
        });
    }
    render() {
        const { loader } = this.config;
        const Loader = loader || DefaultLoader;
        if (this.state.loading || this.state.error) {
            return _jsx(Loader, { ...this.props });
        }
        else if (this.state.result) {
            const Component = this.state.result;
            const Extra = this.config.extra;
            return (_jsxs(_Fragment, { children: [_jsx(Component, { ...this.props }), Extra && _jsx(Extra, {})] }));
        }
        else {
            return null;
        }
    }
}
export function loadable(loadFn, loaderOrConfig) {
    const config = typeof loaderOrConfig === 'object'
        ? loaderOrConfig
        : { loader: loaderOrConfig };
    class LoadableComponent extends LoadableComponentBase {
        static loadFn = loadFn;
        constructor(props) {
            super({
                loadFn,
                ...config,
            }, props);
        }
    }
    return LoadableComponent;
}
